@@ -4,30 +4,25 @@
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE FlexibleContexts #-}
-{-# LANGUAGE Trustworthy #-}
 module Data.Acquire.Internal
     ( Acquire (..)
     , Allocated (..)
     , with
-    , withEx
     , mkAcquire
     , ReleaseType (..)
     , mkAcquireType
     ) where
 
 import Control.Applicative (Applicative (..))
-import Control.Monad.Base (MonadBase (..))
-import Control.Monad.IO.Class (MonadIO (..))
-import Control.Monad.Trans.Control (MonadBaseControl, control)
-import qualified Control.Exception.Lifted as E
+import Control.Monad.IO.Unlift (MonadIO (..), MonadUnliftIO, withRunInIO)
+import qualified Control.Exception as E
 import Data.Typeable (Typeable)
 import Control.Monad (liftM, ap)
-import qualified Control.Monad.Catch as C
-import GHC.IO (unsafeUnmask)
+import qualified Control.Monad.Catch as C ()
 
 -- | The way in which a release is called.
 --
--- Since 1.1.2
+-- @since 1.1.2
 data ReleaseType = ReleaseEarly
                  | ReleaseNormal
                  | ReleaseException
@@ -46,7 +41,7 @@ data Allocated a = Allocated !a !(ReleaseType -> IO ())
 -- implementation in this package is slightly different, due to taking a
 -- different approach to async exception safety.
 --
--- Since 1.1.0
+-- @since 1.1.0
 newtype Acquire a = Acquire ((forall b. IO b -> IO b) -> IO (Allocated a))
     deriving Typeable
 
@@ -69,12 +64,9 @@ instance MonadIO Acquire where
         x <- restore f
         return $! Allocated x (const $ return ())
 
-instance MonadBase IO Acquire where
-    liftBase = liftIO
-
 -- | Create an @Acquire@ value using the given allocate and free functions.
 --
--- Since 1.1.0
+-- @since 1.1.0
 mkAcquire :: IO a -- ^ acquire the resource
           -> (a -> IO ()) -- ^ free the resource
           -> Acquire a
@@ -86,7 +78,7 @@ mkAcquire create free = Acquire $ \restore -> do
 -- cleanup was initiated. This allows you to distinguish, for example, between
 -- normal and exceptional exits.
 --
--- Since 1.1.2
+-- @since 1.1.2
 mkAcquireType
     :: IO a -- ^ acquire the resource
     -> (a -> ReleaseType -> IO ()) -- ^ free the resource
@@ -100,40 +92,13 @@ mkAcquireType create free = Acquire $ \restore -> do
 -- normally or via an exception. This function is similar in function to
 -- @bracket@.
 --
--- Since 1.1.0
-with :: MonadBaseControl IO m
+-- @since 1.1.0
+with :: MonadUnliftIO m
      => Acquire a
      -> (a -> m b)
      -> m b
-with (Acquire f) g = control $ \run -> E.mask $ \restore -> do
+with (Acquire f) g = withRunInIO $ \run -> E.mask $ \restore -> do
     Allocated x free <- f restore
     res <- restore (run (g x)) `E.onException` free ReleaseException
     free ReleaseNormal
     return res
-
--- | Same as @with@, but uses the @MonadMask@ typeclass from exceptions instead
--- of @MonadBaseControl@ from exceptions.
---
--- Since 1.1.3
-#if MIN_VERSION_exceptions(0,6,0)
-withEx :: (C.MonadMask m, MonadIO m)
-#else
-withEx :: (C.MonadCatch m, MonadIO m)
-#endif
-       => Acquire a
-       -> (a -> m b)
-       -> m b
-withEx (Acquire f) g = do
-    -- We need to do some funny business, since the restore we get below is
-    -- specialized to the m from the result, whereas we need a restore function
-    -- in IO. Checking the current masking state is exactly how mask is
-    -- implemented in base.
-    origMS <- liftIO E.getMaskingState
-
-    C.mask $ \restore -> do
-        Allocated x free <- liftIO $ f $ case origMS of
-            E.Unmasked -> unsafeUnmask
-            _ -> id
-        res <- restore (g x) `C.onException` liftIO (free ReleaseException)
-        liftIO $ free ReleaseNormal
-        return res
